#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  4000000		// Xtal speed
__CONFIG(0x3D18);				// Config bits

#define LCD_PORT PORTA			// constants
#define LCD_TRIS TRISA			//
#define LCD_RS 	RA4
#define LCD_RW	RA6
#define LCD_E	RA7

		// Required prototypes.. each function needs to be declared
		// if called BEFORE definition.

void LCD_init(void), LCD_cmd(unsigned char ch);
void LCD_goto(char line, char column), LCD_clr(void);
void LCD_cur(unsigned char ch), pulse_E(void), LCD_hex(int value);
void LCD_char(unsigned char ch), LCD_charD(unsigned char ch);
void convert(int numb);

unsigned char text1[] = {"16 bit counter."};
unsigned char HEX_Table[] = {0x30, 0x31, 0x32, 0x33,
							0x34, 0x35, 0x36, 0x37,
							0x38, 0x39, 0x41, 0x42,
							0x43, 0x44, 0x45, 0x46}; 	//ASCII chracters
unsigned char TenK, Thou, Hund, Tens, Ones;				// Decimal convert routine variables


void main(void)						// program entry
	{
	int index = 0;
	int number = 0;					// new counting variable
	CMCON = 0x7;					// Comparitors off
	LCD_TRIS = 0b00000000;			// Led port as outputs
	__delay_ms(150);				// let LCD stabilise
	LCD_init();						// Initalise screen
	LCD_goto(1,0);					// line 1.
	while(text1[index] != 0)		// while text1[index] is'nt zero
		LCD_char(text1[index++]);	// write message 1

	while(1)						// endless Loop
		{
		LCD_goto(2,2);
		convert(number);			// get decimal representation
		LCD_charD(TenK);			// covert digits
		LCD_charD(Thou);
		LCD_charD(Hund);			// to ascii and
		LCD_charD(Tens);
		LCD_charD(Ones);			// display
		LCD_char(' ');
		LCD_hex(number >>8);		// hex high byte
		LCD_hex(number & 0xff);		// hex low byte
		number++;
		__delay_ms(180);
		}
	}

void convert(int numb)				// Takes a interger number
	{								// and converts to single
	TenK = numb / 10000;			// decimal numbers.
	Thou = (numb % 10000)/ 1000;	//
	Hund = (numb % 1000) / 100;
	Tens = (numb % 100) / 10;
	Ones = numb % 10;
	}

void LCD_init()
	{
	LCD_cmd(0x20);					// 4 bit
	LCD_cmd(0x28);					// display shift
	LCD_cmd(0x6);					// character mode
	LCD_cmd(0xc);					// display on / off and cursor
	LCD_clr();						// clear display
	}


void LCD_hex(int value)
	{
	char data;								// dispay hex values.
	data = value >> 4 & 0xf;				// send upper nibble
	data = HEX_Table[data]; 				// hex lookup
	LCD_char(data);
	data = value & 0xf;						// send lower nibble
	data = HEX_Table[data]; 				// hex lookup
	LCD_char(data);
	}

void LCD_goto(char line, char column)		// combines line and lineW
	{
	unsigned char data = 0x80;				// default to 1
	if(line == 2)data = 0xc0;				// change to 2
	data += column;							// add in  column
	LCD_cmd(data);
	}

void LCD_clr()
	{
	LCD_cmd(1);								// Clr screen
	}

void LCD_cur(char on)
	{
	unsigned char cur = 0xc;				// cursor off
	if(on) cur = 0xd;						// cursor on
	LCD_cmd(cur);
	}

void LCD_cmd(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;			// write high nibble
	LCD_RS = 0;
	pulse_E();
	LCD_PORT = ch & 0xf;				// write low nibble
	LCD_RS = 0;
	pulse_E();
	__delay_ms(5);
	}

void LCD_charD(unsigned char ch)
	{
	ch+=0x30;
	LCD_char(ch);						// convert to ascii
	}

void LCD_char(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;			// High nibble
	LCD_RS = 1;
	pulse_E();
	LCD_PORT = ch & 0xf;				// low nibble
	LCD_RS = 1;
	pulse_E();
	L__delay_ms(5);
	}

void pulse_E()
	{
	LCD_E = 1;
	__delay_us(1);
	LCD_E = 0;
	}